//**************************************************
// Programme de test de la carte CAPAX Xtd
// v1.2 le 28/06/2018
// CAPAX SYSTEMS by INFO-CONSULT
//**************************************************

#include <LiquidCrystal.h>
#include <Keypad.h>
#include <dht.h>

// Déclaration des variables pour l'écran
const int rs=34;
const int en=33;
const int d4=32;
const int d5=31;
const int d6=30;
const int d7=29;
LiquidCrystal lcd(rs,en,d4,d5,d6,d7);
// Fin de la déclaration des variables pour l'écran

// Déclaration de variables pour le clavier (lignes et colonnes)
const byte LIGNES = 4;  
const byte COLONNES = 3;
char touches[LIGNES][COLONNES] = {
  {'1', '2', '3'},
  {'4', '5', '6'},
  {'7', '8', '9'},
  {'*', '0', '#'}
  };
byte BrochesLignes[LIGNES] = {36, 39, 40, 38 };
byte BrochesColonnes[COLONNES] = {37, 35, 41};
Keypad clavier = Keypad( makeKeymap(touches), BrochesLignes, BrochesColonnes, LIGNES, COLONNES );
// Fin de déclaration de variables pour le clavier (lignes et colonnes)

dht DHT;

int v_ldr;
int v_pot;
float v_ldr_a;
int ledState = LOW;


// Déclaration de variables pour la diode RGB
const byte COLOR_BLACK = 0b000;
const byte COLOR_RED = 0b100;
const byte COLOR_GREEN = 0b010;
const byte COLOR_BLUE = 0b001;
const byte COLOR_MAGENTA = 0b101;
const byte COLOR_CYAN = 0b011;
const byte COLOR_YELLOW = 0b110;
const byte COLOR_WHITE = 0b111;
const byte PIN_LED_R = 8;
const byte PIN_LED_G = 12;
const byte PIN_LED_B = 11;
// Déclaration de variables pour la diode RGB

float dmax = 20.0;// si dist >= dmax --> LED verte
float dmin = 10.0; // si dist <= dmin --> LED rouge
int LED_B;
bool b; // etat du bouton poussoir
char InputKey;
int tempC;
float temp;
int DelayColor = 1000;



#define btn 47
#define buzzer 10
#define lm35 A15
#define DHT22_pin A6
#define pot A4
#define echoPin 25
#define trigPin 23
#define ldr A3
#define Led_Vert 42
#define Led_Jaune 44
#define Led_Rouge 43
#define EN 2
#define IN1 3
#define IN2 5
#define Led_Blanche 9
#define CaptHall 18

byte caractere[8] = {
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
  B11111,
};
void setup()
{
  pinMode (lm35, INPUT);
  pinMode(btn, INPUT);
  pinMode(pot, INPUT);
  pinMode(Led_Blanche, OUTPUT);
  pinMode(EN, OUTPUT);
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(buzzer, OUTPUT);
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(PIN_LED_R, OUTPUT);
  pinMode(PIN_LED_G, OUTPUT);
  pinMode(PIN_LED_B, OUTPUT);
  pinMode(Led_Vert, OUTPUT);
  pinMode(Led_Jaune, OUTPUT);
  pinMode(Led_Rouge, OUTPUT);

  lcd.createChar(1, caractere);
  digitalWrite(PIN_LED_R, LOW);
  digitalWrite(PIN_LED_G, LOW);
  digitalWrite(PIN_LED_B, LOW);
  lcd.begin(16, 2);
  lcd.setCursor(0, 0);
  lcd.print("  INFO CONSULT");
  lcd.setCursor(0, 1);
  lcd.print(" Test CAPAX-Xtd");
  delay(2500);
  lcd.clear();

}
void loop()
{
  lcd.setCursor(0, 0);
  lcd.print("Appui sur 1/2/3");
  lcd.setCursor(0, 1);
  lcd.print("4/5/6/7/8/9/0/*");
  
  InputKey = clavier.getKey();

  switch (InputKey) {
    case '1':
      lcd.clear();
      analogReference(INTERNAL1V1); 
  
  // vue que la tension mesurée par le capteur est faible 10mv/°C,
  // ce qui correspond à une tension de 1 V pour 100 °C,
  // il est préférable d'utiliser une tension de référence Vref = 1.1 V au lieu de Vref = 5V */
   
      while (InputKey != '#'){
        PrintLCD('1');
        lcd.setCursor(0, 0);
        lcd.print("Capt. LM35");
        tempC = analogRead(lm35);
        temp = tempC * 0.10752688172; // = 100*1.1 / 1023;
    
        lcd.setCursor(0, 1);
        lcd.print(temp);
        lcd.print(" C  ");
        delay(500);
        InputKey = clavier.getKey();
        } 
      lcd.clear();
      analogReference(DEFAULT);
      break;

    case '2':
      lcd.clear();
      PrintLCD('2');
      while (InputKey != '#'){
        float val = DHT.read22(DHT22_pin);
        float temp = DHT.temperature;
        float hum = DHT.humidity;
        lcd.setCursor(0, 0);
        lcd.print("Capt. DHT 22");
        lcd.setCursor(0, 1);
        lcd.print("T=");
        lcd.print(temp, 1);
        lcd.setCursor(7, 1);
        lcd.print("H=");
        lcd.print(hum, 1);
        InputKey = clavier.getKey();
        } 
      lcd.clear();
      break;

    case '3':
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Capt. HC-SR-04");
      PrintLCD('3');
      lcd.setCursor(0, 1);
      lcd.print("Dist=");
      while (InputKey != '#'){
        float d = ultrason();
        if (d<1000.0){  // Si la distance est > 1000 cm (10 m) la mesure est fausse.
          lcd.setCursor(5, 1);      
          lcd.print(d);
          lcd.print("cm  "); // les 3 " " sont pour effacer l'affichage de "cm" en fonction du nombre de chiffre utilisé pour afficher la distance
            if (d>=dmax) {
              digitalWrite(Led_Vert, HIGH);
              digitalWrite(Led_Jaune, LOW);
              digitalWrite(Led_Rouge, LOW);
              }
            else if (d<=dmin) {
              digitalWrite(Led_Vert, LOW);
              digitalWrite(Led_Jaune, LOW);
              digitalWrite(Led_Rouge, HIGH);
              buzzer3();
              }
            else {
              digitalWrite(Led_Vert, LOW);
              digitalWrite(Led_Jaune, HIGH);
              digitalWrite(Led_Rouge, LOW);
              buzzer1();
              }
            }
      InputKey = clavier.getKey();
      } 
      lcd.clear();
      digitalWrite(Led_Vert, LOW);
      digitalWrite(Led_Jaune, LOW);
      digitalWrite(Led_Rouge, LOW);
      break;

    case '4':
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Diode RGB    4");
      lcd.setCursor(0, 1);
      lcd.print("Rouge");
      displayColor(COLOR_RED);
      delay(DelayColor);
      lcd.setCursor(0, 1);
      lcd.print("Vert ");
      displayColor(COLOR_GREEN);
      delay(DelayColor);
      lcd.setCursor(0, 1);
      lcd.print("Bleu ");
      displayColor(COLOR_BLUE);
      delay(DelayColor);
      lcd.setCursor(0, 1);
      lcd.print("Violet");
      displayColor(COLOR_MAGENTA);
      delay(DelayColor);
      lcd.setCursor(0, 1);
      lcd.print("Bleu clair");
      displayColor(COLOR_CYAN);
      delay(DelayColor);
      lcd.setCursor(0, 1);
      lcd.print("Jaune     ");
      displayColor(COLOR_YELLOW);
      delay(DelayColor);
      lcd.setCursor(0, 1);
      lcd.print("Blanc     ");
      displayColor(COLOR_WHITE);
      delay(DelayColor);
      lcd.clear();
      displayColor(COLOR_BLACK);
      break;

    case '5':
      lcd.clear();
      PrintLCD('5');
      lcd.setCursor(0, 0);
      lcd.print("Appuyer");
      lcd.setCursor(0, 1);
      lcd.print("sur LDR");
      while (InputKey != '#'){
        v_ldr = analogRead(ldr);
        v_ldr_a = float(v_ldr * 5.0 / 1023.0);
        LED_B = max(map(v_ldr, 0, 650, 255, 0),0);
        analogWrite(Led_Blanche, LED_B);
        InputKey = clavier.getKey();
      } 
      analogWrite(Led_Blanche, 0);
      lcd.clear();
      break;

    case '6':
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Test moteur    6");

      lcd.setCursor(0, 1);
      lcd.print("Sens TRIG/V=70");
      digitalWrite (IN1, LOW);
      digitalWrite (IN2, HIGH);
      analogWrite(EN, 70);
      delay(4000);

      lcd.setCursor(12, 1);
      lcd.print("200");// Vitesse = 200
      analogWrite(EN, 200);
      delay(4000);

      lcd.setCursor(0, 1);
      lcd.print("Arret          ");
      analogWrite(EN, 0);
      digitalWrite (IN1, LOW);
      digitalWrite (IN2, LOW);
      delay(4000);

      lcd.setCursor(0, 1);
      lcd.print("Sens HOR/V=150");
      analogWrite(EN, 150);
      digitalWrite (IN1, HIGH);
      digitalWrite (IN2, LOW);
      delay(4000);

      lcd.setCursor(0, 1);
      lcd.print("Arret         ");
      analogWrite(EN, 0);
      digitalWrite (IN1, LOW);
      digitalWrite (IN2, LOW);
      delay(2000);
      lcd.clear();
      break;

    case '7':
      lcd.clear();
      PrintLCD('7');
      lcd.setCursor(0, 0);
      lcd.print("Select. sur M");
      lcd.setCursor(0, 1);
      lcd.print("Tourner pot.");
      while (InputKey != '#'){
        v_pot = analogRead(pot);
        float (v_pot_a) = v_pot * 5 / 1023.0;
        LED_B = map(v_pot, 0, 1023, 0, 255);
        analogWrite(Led_Blanche, LED_B);
        InputKey = clavier.getKey();
      } 
      analogWrite(Led_Blanche, 0);
      lcd.clear();
      break;

    case '8':
       lcd.clear();
       PrintLCD('8');
       lcd.setCursor(0, 0);
       lcd.print("Tourner CD");
       lcd.setCursor(0, 1);
       lcd.print("LED R. clignote");
       attachInterrupt(digitalPinToInterrupt(CaptHall), TimeCount, FALLING);
       while (InputKey != '#'){
         InputKey = clavier.getKey();
         } 
      detachInterrupt(digitalPinToInterrupt(CaptHall));
      digitalWrite(Led_Rouge,LOW);
      lcd.clear();
      break;

    case '9':
    lcd.clear();
    PrintLCD('9');
    lcd.setCursor(0, 0);
    lcd.print("Appuyer sur BP");
    while (InputKey != '#'){
      b = digitalRead (btn);
      digitalWrite(Led_Rouge,!b);
      if (b == LOW) {
        lcd.setCursor(0, 1);
        lcd.print("BP OK");
        }
      else {
        lcd.setCursor(0, 1);
        lcd.print("     ");
        }
      InputKey = clavier.getKey();
      }
      lcd.clear();
      break;

    case '0':
      lcd.clear();
      PrintLCD('0');
      lcd.setCursor(0, 0);
      lcd.print("INFO-CONSULT");
      lcd.setCursor(0, 1);
      lcd.print("CAPAX-Xtd : OK");
      digitalWrite(Led_Vert, HIGH);
      digitalWrite(Led_Jaune, HIGH);
      digitalWrite(Led_Rouge, HIGH);
      analogWrite(Led_Blanche, 10);
  
      while (InputKey != '#'){
        InputKey = clavier.getKey();
        }
      lcd.clear();
      digitalWrite(Led_Vert, LOW);
      digitalWrite(Led_Jaune, LOW);
      digitalWrite(Led_Rouge, LOW);
      analogWrite(Led_Blanche, 0);
      break;

    case '*':
      lcd.clear();
      PrintLCD('*');
      lcd.setCursor(0, 0);
      lcd.print("Touche '*' OK");
      while (InputKey != '#'){
        InputKey = clavier.getKey();
        }
      lcd.clear();
      break;

   }
}



void TimeCount(){
ledState = !ledState;
digitalWrite(Led_Rouge,ledState);
}

void PrintLCD(char k){
    lcd.setCursor(15, 0);
    lcd.print(k);
    lcd.setCursor(15, 1);
    lcd.print("#");
}

float ultrason() {
  float k;
  float dist;
  unsigned long duree;
  unsigned long tmoy;
  unsigned long tmin = 3000000; // le temps maximal d'attente de l'impulsion. pour plus de détail, consulter l'aide de pulseIn dans www.arduino.cc
  unsigned long tmax = 0;
  k = 0.01717; // k=343.3/20000 = vitesse des US dans l'air / temps en [microseconde] relatif à un aller et retour de l'onde. La distance est ainsi calculée en [cm]
  for (int i = 0; i < 5; i++) // pour une meilleure précision de la distance, on calcule la moyenne de 5 valeurs.
  {
    digitalWrite(trigPin, LOW);
    delayMicroseconds(2);
    digitalWrite(trigPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(trigPin, LOW);
    duree = pulseIn(echoPin, HIGH);
    if (duree > tmax){
      tmax = duree;
    }
    else if (duree < tmin){
      tmin = duree;
    }
    delay(50);
    }
    tmoy = tmax/2 + tmin / 2;
// ne pas calculer tmoy= (tmax+tmin)/2, le calcul de l'addition en premier lieu 
// peut provoquer un dépassement de capacité selon le type de la variable.
    dist = tmoy * k;
    return dist;
}

void displayColor(byte color){
  digitalWrite(PIN_LED_R, bitRead(color, 2));
  digitalWrite(PIN_LED_G, bitRead(color, 1));
  digitalWrite(PIN_LED_B, bitRead(color, 0));
}

void buzzer2 (){
  tone(buzzer, 989, 1000);
  delay(500);

  tone(buzzer, 100, 1000);
  delay(500);
}
void buzzer3 (){
  tone(buzzer, 989, 1000);
  delay(100);

  tone(buzzer, 100, 1000);
  delay(100);
}
void buzzer1(){
  tone(buzzer, 989, 1000);
  delay(700);

  tone(buzzer, 100, 1000);
  delay(700);
}
